<?php
/**
 * Pluggable functions for boards
 * @since 1.5.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Add the page title
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_board_tax_heading' ) ) {
	function ctdb_board_tax_heading() {
		$board = single_term_title( '', false ); ?>
		<header class="entry-header ctdb-board-tax-header">
			<?php printf( '<h1 class="%s">%s</h1>',
				'entry-title ctdb-page-title',
				 $board
			 ); ?>
		</header>
		<?php
	}
	add_action ( 'ctdb_board_header', 'ctdb_board_tax_heading' );
}

/**
 * Add the sidebar before the footer
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_board_sidebar' ) ) {
	function ctdb_board_sidebar() {
		if( is_tax( 'board' ) ) {
			get_sidebar();
		}
	}
	add_action ( 'ctdb_close_wrapper_archive', 'ctdb_board_sidebar', 20 );
}

/**
 * Open the wrapper around the content
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_open_board_tax_content' ) ) {
	function ctdb_open_board_tax_content() {
		echo '<div class="entry-content">';
	}
	add_action ( 'ctdb_board_content', 'ctdb_open_board_tax_content', 5 );
}

/**
 * Add the shortcode
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_board_tax_content' ) ) {
	function ctdb_board_tax_content() {
		// Get the taxonomy ID
		$current_tax = get_query_var( 'board' );
		$term = get_term_by( 'slug', $current_tax, 'board' );
		$term_id = $term->term_id;
		
		echo '<div class="ctdb-board-outer">';
		echo '<div class="ctdb-board-wrapper">';
		// echo ctdb_get_board_header( $term_id, $term );
		echo ctdb_get_board_meta( $term_id, $term );
		echo '</div>';
		
		// Add a filter / action here to specify cols
		// Get selected fields to display
		$options = get_option( 'ctdb_boards_settings' );
		$params = array(); // Add our shortcode atts here
		$fields = array();
		if( isset( $options['boards_meta_fields_archive'] ) ) {
			$fields = $options['boards_meta_fields_archive'];
			// Use this filter to check legitimacy of fields, e.g. if Status is selected but not available for a certain Board
			$fields = apply_filters( 'ctdb_meta_data_fields_filter', $fields );
			if( ! empty( $fields ) ) {
				$cols = 'avatar,' . join( ',', $fields );
				$params[] = 'cols="' . $cols . '"';
			}
		}
		
		// Add orderby and order parameters
		if( isset( $options['boards_orderby'] ) ) {
			$params[] = 'orderby="' . esc_attr( $options['boards_orderby'] ) . '"';
		}
		if( isset( $options['boards_order'] ) ) {
			$params[] = 'order="' . esc_attr( $options['boards_order'] ) . '"';
		}
		
		echo do_shortcode( '[discussion_topics ' . join( ' ', $params ) . ' board=' . $term_id . ']' );
		
		echo ctdb_get_board_footer( $term_id, $term );
		
		echo '</div>';
	}
	add_action ( 'ctdb_board_content', 'ctdb_board_tax_content', 10 );
}

/**
 * Close the wrapper around the content
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_close_board_tax_content' ) ) {
	function ctdb_close_board_tax_content() {
		echo '</div>';
	}
	add_action ( 'ctdb_board_content', 'ctdb_close_board_tax_content', 100 );
}

/**
 * Return the board header for the discussion_board shortcode
 * @since 1.7.0
 */
if( ! function_exists( 'ctdb_get_board_header' ) ) {
	function ctdb_get_board_header( $board_id, $term ) {
		$content = '<div class="ctdb-board-header">';

			$content .= '<div class="ctdb-board-header-inner">';
			if( ctdb_is_user_permitted() == true && $term->count > 0 ) {
				// Site title and link
				$content .= sprintf( '<h2 class="%s"><a href="%s">%s</a></h2>',
					esc_attr( 'board-title' ),
					esc_url( get_term_link( $board_id ) ),
					esc_textarea( $term->name ) 
				);
			} else {
				// Site title only if no topics exist yet or user is not permitted
				$content .= sprintf( '<h2 class="%s">%s</h2>',
					esc_attr( 'board-title' ),
					esc_textarea( $term->name ) 
				);
			}
			$content .= '</div><!-- ctdb-board-header-inner -->';
	
		$content .= '</div><!-- ctdb-board-header -->';
		
		return $content;
	}
}

/**
 * Return the board meta for the discussion_board shortcode
 * @since 1.7.0
 */
if( ! function_exists( 'ctdb_get_board_meta' ) ) {
	function ctdb_get_board_meta( $board_id, $term ) {
		$content = '<div class="ctdb-board-meta-wrap">';
			$content .= '<div class="ctdb-board-thumb">';
				$image_id = get_term_meta( $term->term_id, 'ctdb-board-image-id', true );
				$content .= wp_get_attachment_image( $image_id );
			//	$content .= wpautop( $term->description );
			$content .= '</div><!-- ctdb-board-thumb -->';
			$content .= '<div class="ctdb-board-description">';
				$content .= wpautop( $term->description );
			$content .= '</div><!-- ctdb-board-description -->';
			$content .= '<div class="ctdb-board-meta">';
				// Get selected fields to display
				$options = get_option( 'ctdb_boards_settings' );
				$fields = array();
				if( isset( $options['boards_fields'] ) ) {
					$fields = $options['boards_fields'];
				}

				// Get list of all available fields
				$all_fields = ctdb_boards_fields();

				// Iterate through selected fields
				if( ! empty( $fields ) && ! empty( $all_fields ) ) {
					foreach( $fields as $field ) {
	
						// If the field is available, add the content
						if( isset( $all_fields[$field]['callback'] ) && function_exists( $all_fields[$field]['callback'] ) ) {
							$content .= '<div class="ctdb-board-meta-inner ctdb-board-meta-' . esc_attr( $field ) . '">';
								$label = $all_fields[$field]['label'];
								$content .= call_user_func( $all_fields[$field]['callback'], $term, $term->term_id, $label );
							$content .= '</div>';
						}
	
					}
				}
			$content .= '</div><!-- ctdb-board-meta -->';
		$content .= '</div><!-- ctdb-board-meta-wrap -->';
		
		return $content;
	}
}

/**
 * Get the board footer
 * @since 1.7.0
 * @param $board_id
 * @param $term
 */
function ctdb_get_board_footer( $board_id, $term ) {
	
	// global $CT_DB_Public;

	$footer = '';
	
	if( ctdb_is_user_permitted() == true ) {

		$footer = apply_filters( 'ctdb_filter_board_footer', $footer, $board_id, $term );

		// If we haven't already built the footer content, then build it below
		if( empty( $footer ) ) {
	
			$footer .= '<div class="ctdb-board-topics-link">';
	
				if( isset( $term->count ) && $term->count > 0 ) {
					// Only add the link if there are topics in the term
					$footer .= sprintf( '<div class="ctdb-board-topics-link-inner"><a href="%s">%s</a></div>',
						esc_url( get_term_link( $board_id ) ), 
						__( 'See all topics', 'discussion-board-pro' )
					);
				}				
	
				// Get the page with the new topic form
				$options = get_option( 'ctdb_options_settings' );
				if( isset( $options['new_topic_page'] ) && $options['new_topic_page'] != '' ) {
					if( ctdb_is_posting_permitted_by_board( $board_id ) ) {
						// If the user is permitted to post on this board, add a link
						$new_topic_page = intval( $options['new_topic_page'] );
						$footer .= sprintf( '<div class="ctdb-board-topics-link-inner"><a href="%s">%s</a></div>',
							esc_url( add_query_arg(
								array(
									'board_id' => $board_id
								), get_permalink( $new_topic_page ) )
							), 
							__( 'Add new topic', 'discussion-board-pro' )
						);
					}
				} else if( current_user_can( 'update_plugins' ) ) {
					// Display a message to admins that there's no topic page set
					// Show admin a message to let them know
					$footer .= '<p class="ctdb-admin-message">' . __( 'Site Admin: please set the "New topic form page" field in Settings > Discussion Board > General for the new_topic_button shortcode to work. This message will not be displayed to non-admins.', 'discussion-board-pro' ) . '</p>';
				}
		
				// Add a follow board option if enabled
				$follower_options = get_option( 'ctdb_followers_settings' );
				if( ! empty( $follower_options['enable_board_following'] ) && is_user_logged_in() ) {
					// Is the user following this board?
					$is_following = ctdb_is_following_board( $board_id );
					$footer .= '<div class="ctdb-board-topics-link-inner">';
					if( $is_following ) {
						$footer .= sprintf( '<a id="ctdb-follow-board-%s" class="ctdb-follow-board" data-board="%s" data-action="unfollow" href="#">%s</a>',
							absint( $board_id ),
							absint( $board_id ),
							ctdb_get_unfollow_this_board_message()
						);
					} else {
						$footer .= sprintf( '<a id="ctdb-follow-board-%s" class="ctdb-follow-board" data-board="%s" data-action="follow" href="#">%s</a>',
							absint( $board_id ),
							absint( $board_id ),
							ctdb_get_follow_this_board_message()
						);
					}
					$footer .= wp_nonce_field( 'board_follow_nonce_' . absint( $board_id ), 'board_follow_nonce_' . absint( $board_id ), true, false );
					$footer .= '</div><!-- .ctdb-board-topics-link-inner -->';
			
				}
	
			$footer .= '</div><!-- .ctdb-board-topics-link -->';
	
		}

	}
	
	return $footer;
	
}

/**
 * Return the available boards fields to display on the Boards summary
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_boards_fields' ) ) {
	function ctdb_boards_fields() {
		$fields = array( 
			'topic'		=> array(
				'id'		=> 'topic',
				'label'		=> __( 'Topics', 'discussion-board-pro' ),
				'callback'	=> 'ctdb_boards_topics_callback'
			),
			'replies'		=> array(
				'id'		=> 'replies',
				'label'		=> __( 'Replies', 'discussion-board-pro' ),
				'callback'	=> 'ctdb_boards_replies_callback'
			),
			'freshness'			=> array(
				'id'		=> 'freshness',
				'label'		=> __( 'Freshness', 'discussion-board-pro' ),
				'callback'	=> 'ctdb_boards_freshness_callback'
			),
			'voices'			=> array(
				'id'		=> 'voices',
				'label'		=> __( 'Voices', 'discussion-board-pro' ),
				'callback'	=> 'ctdb_boards_voices_callback'
			),
		);
		$fields = apply_filters( 'ctdb_boards_fields', $fields );
		return $fields;
	}
}

/***
 * Return the available boards fields to display on the Boards archive
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_boards_fields_archive' ) ) {
	function ctdb_boards_fields_archive() {
		$fields = array( 
			'topic'		=> array(
				'id'		=> 'topic',
				'label'		=> __( 'Topics', 'discussion-board-pro' ),
				'callback'	=> 'ctdb_boards_topics_callback'
			),
			'replies'		=> array(
				'id'		=> 'replies',
				'label'		=> __( 'Replies', 'discussion-board-pro' ),
				'callback'	=> 'ctdb_boards_replies_callback'
			),
			'started'		=> array(
				'id'		=> 'started',
				'label'		=> __( 'Started', 'discussion-board-pro' ),
				'callback'	=> 'ctdb_boards_started_callback'
			),
			'freshness'			=> array(
				'id'		=> 'freshness',
				'label'		=> __( 'Freshness', 'discussion-board-pro' ),
				'callback'	=> 'ctdb_boards_freshness_callback'
			),
			'voices'			=> array(
				'id'		=> 'voices',
				'label'		=> __( 'Voices', 'discussion-board-pro' ),
				'callback'	=> 'ctdb_boards_voices_callback'
			),
			'status'			=> array(
				'id'		=> 'status',
				'label'		=> __( 'Status', 'discussion-board-pro' ),
				'callback'	=> 'ctdb_boards_status_callback'
			),
		);
		$fields = apply_filters( 'ctdb_boards_fields', $fields );
		return $fields;
	}
}

/**
 * Return the markup for number of topics within a Board
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_boards_topics_callback' ) ) {
	function ctdb_boards_topics_callback( $term, $term_id, $label ) {
		$return = '<div class="ctdb-board-meta-left">' . $label . ': </div>';
		$return .= '<div class="ctdb-board-meta-right">' . ctdb_get_board_topics_count( $term ) . '</div>';
		return $return;
	}
}

/**
 * Return the number of topics within a Board
 * @since 1.7.0
 */
if( ! function_exists( 'ctdb_get_board_topics_count' ) ) {
	function ctdb_get_board_topics_count( $term ) {
		return $term->count;
	}
}

/**
 * Return the number of comments within a Board
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_boards_replies_callback' ) ) {
	function ctdb_boards_replies_callback( $term, $term_id, $label ) {
		$return = '<div class="ctdb-board-meta-left">' . $label . ': </div>';
		$return .= '<div class="ctdb-board-meta-right">' . ctdb_get_board_replies_count( $term_id ) . '</div>';
		return $return;
	}
}

/**
 * Return the number of replies within a Board
 * @since 1.7.0
 */
if( ! function_exists( 'ctdb_get_board_replies_count' ) ) {
	function ctdb_get_board_replies_count( $term_id ) {
		// We have to count all topics
		$args = array(
			'post_type'			=> 'discussion-topics',
			'posts_per_page'	=> -1,
			'tax_query'			=> array(
				array(
					'taxonomy'	=> 'board',
					'field'		=> 'term_id',
					'terms'		=> $term_id,
				)
			)
		);
		
		global $post;
		$replies = new WP_Query( $args );
		$count = 0;
		if( $replies->have_posts() ) {
			while( $replies->have_posts() ) : $replies->the_post();
				$count += get_comments_number( $post->ID );
			endwhile;
		}
		
		return $count;
	}
}

/**
 * Return the date and time of the last activity in the Board
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_boards_freshness_callback' ) ) {
	function ctdb_boards_freshness_callback( $term, $term_id, $label ) {
		// Query the topics in the Board, latest one first
		$freshness = ctdb_get_board_freshness( $term_id );
		
		$return = '<div class="ctdb-board-meta-left">' . $label . ': </div>';
		if( $freshness ) {
			$return .= '<div class="ctdb-board-meta-right">' . $freshness . '</div>';
		}
		
		return $return;
	}
}

/**
 * Return the date and time of the last activity in the Board
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_get_board_freshness' ) ) {
	function ctdb_get_board_freshness( $term_id ) {
		// Query the topics in the Board, latest one first
		$args = array(
			'post_type'			=> 'discussion-topics',
			'posts_per_page'	=> 1,
			'orderby' 			=> 'meta_value_num',
			'meta_key'			=> 'ctdb_most_recent_activity',
			'tax_query'			=> array(
				array(
					'taxonomy'	=> 'board',
					'field'		=> 'term_id',
					'terms'		=> $term_id,
				)
			)
		);
		global $post;
		
		$replies = new WP_Query( $args );

		if( $replies->have_posts() ) {
			while( $replies->have_posts() ) : $replies->the_post();
				$date = get_post_meta( $post->ID, 'ctdb_most_recent_activity', true );
			endwhile;
		}
		if( isset( $date ) ) {
			$date_format = get_option( 'date_format' );
			$latest = date( $date_format, $date );
			$freshness = '<time class="timeago" datetime="' . esc_attr( date( 'c', $date ) ) . '">' . esc_html( $latest ) . '</time>';
		} else {
			$date = '';
			$latest = '';
			$freshness = '';
		}
		
		
		
		return $freshness;
	}
}



/**
 * Return the number of voices in a Board
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_boards_voices_callback' ) ) {
	function ctdb_boards_voices_callback( $term, $term_id, $label ) {	
		$return = '<div class="ctdb-board-meta-left">' . $label . ': </div>';
		$return .= '<div class="ctdb-board-meta-right">' . ctdb_get_board_voices_count( $term_id ) . '</div>';
		return $return;
	}
}

/**
 * Return the number of voices in a Board
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_get_board_voices_count' ) ) {
	function ctdb_get_board_voices_count( $term_id ) {
		
		// We have to count all topics
		$args = array(
			'post_type'			=> 'discussion-topics',
			'posts_per_page'	=> -1,
			'tax_query'			=> array(
				array(
					'taxonomy'	=> 'board',
					'field'		=> 'term_id',
					'terms'		=> $term_id,
				)
			)
		);
		
		global $post;
		$topics = new WP_Query( $args );
		// Count commenters by email
		$commenters = array();
		
		if( $topics->have_posts() ) {
			while( $topics->have_posts() ) : $topics->the_post();
				// Add the topic poster to the list
				if( ! in_array( get_the_author_meta( 'email' ), $commenters ) ) {
					$commenters[] = get_the_author_meta( 'email' );
				}
				// Then add any commenters
				$comments = get_comments( array( 'post_id' => $post->ID ) );
				if( ! empty( $comments ) ) {
					foreach( $comments as $comment ) {
						if( ! in_array( $comment->comment_author_email, $commenters ) ) {
							$commenters[] = $comment->comment_author_email;
						}
					}
				}
			endwhile;
		}
		if( ! empty( $commenters ) ) {
			$total = count( $commenters );
		} else {
			$total = 0;
		}
		
		return $total;
	}
}



/**
 * Return the topic status within a Board
 * @since 1.5.0
 */
if( ! function_exists( 'ctdb_boards_topics_callback' ) ) {
	function ctdb_boards_status_callback( $term, $term_id, $label ) {
		$return = '<div class="ctdb-board-meta-left">' . $label . ': </div>';
		$return .= '<div class="ctdb-board-meta-right">' . get_the_ID() . '</div>';
		return $return;
	}
}

/**
 * Return the Board ID for a topic
 * @since 1.6.0
 */
if( ! function_exists( 'ctdb_topic_board_id' ) ) {
	function ctdb_topic_board_id( $post_id=false ) {
		$board_id = '';
		if( false === $post_id ) {
			$post_id = get_the_ID();
		}
		$terms = wp_get_post_terms( $post_id, 'board', array( 'fields' => 'ids' ) );
		if( ! empty ( $terms ) && is_array( $terms ) ) {
			$board_id = $terms[0];
		}
		return $board_id;
	}
}

/**
 * Return the ID of the main Board page
 * @since 1.6.0
 */
if( ! function_exists( 'ctdb_board_page_id' ) ) {
	function ctdb_board_page_id() {
		$options = get_option( 'ctdb_options_settings' );
		if( isset( $options['board_page'] ) ) {
			return absint( $options['board_page'] );
		}
		return null;  
	}
}

/**
 * Return the ID of the Discussion Topics page
 * @since 1.6.2
 */
if( ! function_exists( 'ctdb_discussion_page_id' ) ) {
	function ctdb_discussion_page_id() {
		$options = get_option( 'ctdb_options_settings' );
		if( isset( $options['discussion_topics_page'] ) ) {
			return absint( $options['discussion_topics_page'] );
		}
		return null;  
	}
}

/**
 * Are boards enabled?
 * @since 1.6.2
 */
if( ! function_exists( 'ctdb_are_boards_enabled' ) ) {
	function ctdb_are_boards_enabled() {
		$options = get_option( 'ctdb_boards_settings' );
		if( ! empty( $options['enable_boards'] ) ) {
			return true;
		}
		return false;
	}
}

/**
 * Is the user following this board?
 * @since 1.6.2
 */
if( ! function_exists( 'ctdb_is_following_board' ) ) {
	function ctdb_is_following_board( $board_id ) {
		// If boards aren't enabled, don't worry about the rest of it
		if( ! ctdb_are_boards_enabled() ) {
			return;
		}
		$followers = get_term_meta( $board_id, 'ctdb_board_followers', true );
		if( ! empty( $followers ) ) {
			$followers = explode( ',', $followers );
			// If we've got some followers, let's check if the current user is in the list
			$current_user = wp_get_current_user();
			if( $current_user && isset( $current_user->user_email ) ) {
				if( in_array( $current_user->user_email, $followers ) ) {
					// The current user is following this board
					return true;
				}
			}
		}
		// The current user is not following the board
		return false;
		
	}
}

/**
 * Are follower boards enabled?
 * @since 1.6.2
 */
if( ! function_exists( 'ctdb_are_follower_boards_enabled' ) ) {
	function ctdb_are_follower_boards_enabled() {
		$follower_options = get_option( 'ctdb_followers_settings' );
		if( ! empty( $follower_options['enable_board_following'] ) ) {
			return true;
		}
		return false;
	}
}


/**
 * Return followers for specific board
 * @since 1.6.2
 * @param	$board_id	Board ID to get followers of
 * @param	$exclude	Array of values to exclude from follower list, e.g. post author email, comment author email
 * @return	Array		$follower_list
 */
if( ! function_exists( 'ctdb_get_board_followers' ) ) {
	function ctdb_get_board_followers( $board_id, $exclude ) {
		if( ! $board_id ) {
			return;
		}
		$follower_list = array();
		// Get the board followers
		$board_followers = get_term_meta( $board_id, 'ctdb_board_followers', true );
		if( ! empty( $board_followers ) ) {
			// Split into array
			$board_followers = explode( ',', $board_followers );
			if( is_array( $board_followers ) ) {
				foreach( $board_followers as $board_follower ) {
					$board_follower = trim( $board_follower );
					// Check we're not including any excluded emails
					if( $board_follower != $exclude ) {
						$follower_list[] = $board_follower;
					}
				}
			}
		}
		return $follower_list;
	}
}

/**
 * Return possible permission settings for boards
 * @since 1.7.0
 * @return	Array		$follower_list
 */
if( ! function_exists( 'ctdb_get_board_permission_options' ) ) {
	function ctdb_get_board_permission_options() {
		return apply_filters(
			'ctdb_filter_board_permissions',
			array(
				'default'		=> __( 'Default', 'discussion-board-pro' ),
				'author-only'	=> __( 'Author Only', 'discussion-board-pro' ),
				'post-only'		=> __( 'Post Only', 'discussion-board-pro' ),
			)
		);
	}
}

/**
 * Get the permission setting for a topic's board
 * @since 1.7.0
 * @return	String
 */
if( ! function_exists( 'ctdb_get_board_permission' ) ) {
	function ctdb_get_board_permission( $post_id ) {
		$terms = get_the_terms( $post_id, 'board' );
		if( ! is_wp_error( $terms ) && isset( $terms[0]->term_id ) ) {
			$board_id = $terms[0]->term_id;
			$permission = get_term_meta( $board_id, 'ctdb-board-permissions', true );
			return $permission;
		}
		return 'default';
	}
}

/**
 * Check if the board has any special permissions et
 * @since 1.7.0
 * @return	String
 */
if( ! function_exists( 'ctdb_filter_single_content_for_permissions' ) ) {
	function ctdb_filter_single_content_for_permissions( $content, $position ) {
		global $post;
		if( 'discussion-topics' != get_post_type() ) {
			return $content;
		}
		$permission = ctdb_get_board_permission( $post->ID );
		if( $permission == 'post-only' ) {
			// Don't display a comment form
			add_action( 'ctdb_empty_comment_file', 'ctdb_comment_message_post_only' );
			
		} else if( $permission == 'author-only' ) {
			// Only the author can view replies
			$author_id = get_the_author_meta( 'ID' );
			$user_id = get_current_user_id();
			if( $author_id != $user_id ) {
				// Return a message instead of the content
				$content .= sprintf(
					'<p class="author-only-message">%s</p>',
					apply_filters( 
						'ctdb_filter_author_only_message',
						__( '"Replies to topics on this board can only be viewed by the original author of the topic', 'discussion-board-pro' )
					)
				);
			}
		}
		
		return $content;
	}
}
add_filter( 'ctdb_filter_single_content_end', 'ctdb_filter_single_content_for_permissions', 1000, 2 );


function ctdb_comment_message_post_only() {
	printf(
		'<p>%s</p>',
		apply_filters( 
			'ctdb_filter_post_only_message',
			__( 'This topic is post only and is closed to replies', 'discussion-board-pro' )
		)
	);
}

function ctdb_comment_message_author_only( $content ) {
	$content .= sprintf(
		'<p>%s</p>',
		apply_filters( 
			'ctdb_filter_author_only_message',
			__( 'Only the topic author can view replies', 'discussion-board-pro' )
		)
	);
}


/**
 * Filter the comment form file if the topic is post only or if it's author only and current user is not the author
 * @since 1.7.0
 * @return	Path
 */
function ctdb_filter_comments_for_permissions( $comment_template ) {
	global $post;
	// If the user doesn't have access to view, return an empty file for the comments template
	if( is_single() && 'discussion-topics' == get_post_type() ) {
		$permission = ctdb_get_board_permission( $post->ID );
		if( $permission == 'post-only' ) {
			// Don't display a comment form
			$comment_template = ctdb_get_comments_file_path();
		} else if( $permission == 'author-only' ) {
			// Only the author can view replies
			$author_id = get_the_author_meta( 'ID' );
			$user_id = get_current_user_id();
			if( $author_id != $user_id ) {
				// Don't display replies if you're not the author
				add_filter( 'wp_list_comments_args', 'ctdb_filter_wp_list_comments_args' );
			}
		}
	}
	return $comment_template;
}
add_filter( 'comments_template', 'ctdb_filter_comments_for_permissions' );

/**
 * Filter comment list args so comments don't display
 * @since 1.7.0
 * @return	Array
 */
function ctdb_filter_wp_list_comments_args( $r ) {
	$r['echo'] = false;
	return $r;
}

/**
 * Filter permitted viewer roles
 * @since 1.8.0
 * @return	Array
 */
function ctdb_filter_permitted_viewer_roles_by_board( $viewer_roles ) {
	$plan = ctdb_get_license_plan();
	if( false === $plan || $plan == 'basic' ) {
		// We can only do board-specific permissions on developer and unlimited plans
		return $viewer_roles;
	}
	$board_id = ctdb_topic_board_id();
	if( $board_id ) {
		// Check for different viewer roles on this board
		$board_viewer_roles = get_term_meta( $board_id, 'viewer_roles', true );
		
		if( ! empty( $board_viewer_roles ) && is_array( $board_viewer_roles ) ) {
			$viewer_roles = $board_viewer_roles;
			// Admins can always view
			$viewer_roles[] = 'administrator';
		}

	}
	return $viewer_roles;
}
add_filter( 'ctdb_filter_permitted_viewer_roles', 'ctdb_filter_permitted_viewer_roles_by_board' );

/**
 * Filter permitted viewer roles
 * @since 1.8.0
 * @return	Array
 */
function ctdb_filter_permitted_poster_roles_by_board( $poster_roles ) {
	$plan = ctdb_get_license_plan();
	if( false === $plan || $plan == 'basic' ) {
		// We can only do board-specific permissions on developer and unlimited plans
		return $poster_roles;
	}
	$board_id = ctdb_topic_board_id();
	if( $board_id ) {
		// Check for different viewer roles on this board
		$board_poster_roles = get_term_meta( $board_id, 'poster_roles', true );
		
		if( ! empty( $board_poster_roles ) && is_array( $board_poster_roles ) ) {
			$poster_roles = $board_poster_roles;
		}

	}
	return $poster_roles;
}
add_filter( 'ctdb_filter_permitted_poster_roles', 'ctdb_filter_permitted_poster_roles_by_board' );

/**
 * Gets whether a user can post to a specific board
 * @param $board_id
 * @since 1.8.0
 * @return Boolean
 */
function ctdb_is_posting_permitted_by_board( $board_id ) {
	$can_post = false;
	$role = ctdb_get_role();
	
	// Check against board specific rules first
	$plan = ctdb_get_license_plan();
	if( $plan == 'developer' || $plan == 'unlimited' ) {
		// We can only do board-specific permissions on developer and unlimited plans
		$poster_roles = get_term_meta( $board_id, 'poster_roles', true );
	}
	
	// If no board roles set, get global poster roles
	if( empty( $poster_roles ) ) {
		$poster_roles = ctdb_get_unfiltered_permitted_poster_roles();
	}
	
	// If it's still empty, anyone can post
	if( empty( $poster_roles ) ) {
		$can_post = true;
	} else if( is_array( $poster_roles ) && ! empty( $role ) && in_array( $role[0], $poster_roles ) ) {
		$can_post = true;
	}
	
	return $can_post;
}

/**
 * Filter posting permission on new topic page
 * @param $user_do_post
 * @since 1.8.0
 * @return Boolean
 */
function ctdb_filter_is_posting_permitted( $user_do_post ) {
	$plan = ctdb_get_license_plan();
	if( false === $plan || $plan == 'basic' ) {
		// We can only do board-specific permissions on developer and unlimited plans
		return $user_do_post;
	}
	if( isset( $_GET['board_id'] ) ) {
		$board_id = $_GET['board_id'];
		// Check if user can post to this board
		$user_do_post = ctdb_is_posting_permitted_by_board( $board_id );
	}
	return $user_do_post;
}
add_filter( 'ctdb_filter_is_posting_permitted', 'ctdb_filter_is_posting_permitted' );